@@ -7,9 +7,13 @@ use utf8;
 use base qw( Exporter );
 
 our @EXPORT = qw();
-our @EXPORT_OK = qw( key_exists );
+our @EXPORT_OK = qw( 
+        key_exists 
+        every_keys 
+        some_keys 
+);
 our %EXPORT_TAGS = ( all => [ @EXPORT, @EXPORT_OK ] );
-our $VERSION = '0.02';
+our $VERSION = '0.2';
 
 sub key_exists($$) {
     my $hash_ref = shift;
@@ -20,7 +24,7 @@ sub key_exists($$) {
     my $key = shift @$array_ref;
     
     if ( exists $hash_ref->{$key} ) {
-        if ( scalar(@$array_ref) > 0 ) {
+        if ( scalar @$array_ref > 0 ) {
             return &key_exists( $hash_ref->{$key}, $array_ref );
         } else {
             return 1;
@@ -30,6 +34,28 @@ sub key_exists($$) {
     }#fi
 }
 
+sub every_keys($$) {
+    my $hash_ref = shift;
+    my $list_array_ref = shift;
+    
+    foreach my $array_ref ( @$list_array_ref ) {
+        return 0 unless key_exists( $hash_ref, $array_ref );
+    }#hcaerof
+    
+    return 1;
+}
+
+sub some_keys($$) {
+    my $hash_ref = shift;
+    my $list_array_ref = shift;
+    
+    foreach my $array_ref ( @$list_array_ref ) {
+        return 1 if key_exists( $hash_ref, $array_ref );
+    }#hcaerof
+    
+    return 0;
+}
+
 1;
 
 
@@ -38,11 +64,11 @@ __END__
 
 =head1 NAME
 
-Deep::Hash::Exists - verify existence of keys hash
+Deep::Hash::Exists - Verify existence of keys hash.
 
 =head1 SYNOPSIS
 
- use Deep::Hash::Exists qw( key_exists );
+ use Deep::Hash::Exists ':all';
  
  my $hash_ref = {
          A => 'one',
@@ -53,15 +79,14 @@ Deep::Hash::Exists - verify existence of keys hash
          },
  };
  
- if ( key_exists( $hash_ref, [ 'C', 'one' ] ) ) {
-     printf "Output: %d", key_exists( $hash_ref, [ 'C', 'one' ] );
- }
- # Output: 1
+ key_exists( $hash_ref, [ 'C', 'one' ] ); # return 1
+ key_exists( $hash_ref, [ 'D' ] );        # return 0
  
- if ( key_exists( $hash_ref, [ 'D' ] ) ) {
-     printf "Output: %d", key_exists( $hash_ref, [ 'D' ] );
- }
- # Output: 0
+ every_keys( $hash_ref, [ ['A'], ['B', 0], ['C', 'one'] ] ); # return 0
+ every_keys( $hash_ref, [ ['A'], ['B'], ['C', 'one'] ] );    # return 1
+ 
+ some_keys( $hash_ref, [ ['A'], ['B', 0], ['C', 'one'] ] ); # return 1
+ some_keys( $hash_ref, [ ['D'], ['B', 0], ['C', 'six'] ] ); # return 0
  
 =head1 DESCRIPTION
 
@@ -120,12 +145,11 @@ Output:
 
 =head2 key_exists($$)
 
-B<key_exists>( I<$hash_ref, $array_ref_keys> ) - verify existence of keys hash
+B<key_exists>( I<$hash_ref, $array_ref_keys> ) - Verify existence of keys hash.
 
 B<Return>:
 
-    1 - if exists hash key
-    0 - if not exists hash key
+    If exist keys of hash then return 1, otherwise - 0.
 
 B<Example>:
  
@@ -155,6 +179,60 @@ B<Example>:
 =cut
 
 
+=head2 every_keys($$)
+
+B<every_keys>( I<$hash_ref, $list_array_ref> ) - Verify list of hash keys for existence.
+
+B<Return>:
+
+    If exist all keys of hash of the submitted list then return 1, otherwise - 0.
+
+B<Example>:
+
+ my $hash_ref = {
+         A => 'one',
+         B => [ 'one', 'two' ],
+         C => { 
+             'one' => 1, 
+             'two' => 2, 
+         },
+ };
+ 
+ printf "Output: %s", every_keys( $hash_ref, [ ['A'], ['B', 0], ['C', 'one'] ] );
+ # Output: 0
+ printf "Output: %s", every_keys( $hash_ref, [ ['A'], ['C', 'one'], ['C', 'two'] ] );
+ # Output: 1
+
+=cut
+
+
+=head2 some_keys($$)
+
+B<some_keys>( I<$hash_ref, $list_array_ref> ) - Verify list of hash keys for existence.
+
+B<Return>:
+
+    If exist at least one key of hash of the submitted list then return 1, otherwise - 0.
+
+B<Example>:
+
+ my $hash_ref = {
+         A => 'one',
+         B => [ 'one', 'two' ],
+         C => { 
+             'one' => 1, 
+             'two' => 2, 
+         },
+ };
+ 
+ printf "Output: %s", some_keys( $hash_ref, [ ['A'], ['B', 0], ['C', 'one'] ] );
+ # Output: 1
+ printf "Output: %s", some_keys( $hash_ref, [ ['A', 'one'], ['B', 0], ['C', 'three'] ] );
+ # Output: 0
+
+=cut
+
+
 =head1 SEE ALSO
 
 L<Hash::Util>, L<Deep::Hash::Utils>
